class UserValidity:
    # Not registered (0)
    Unregistered = "Unregistered"
    # Registered, but not valid for licensed purchase (1)
    Partial = "Partial"
    # Registered and valid for licensed purchase (2)
    Full = "Full"
    # Barred, not able to purchase (3)
    Barred = "Barred"
